@@ -2,6 +2,8 @@
 
 from __future__ import with_statement
 
+from uuid import uuid4
+
 # Enthought library imports
 from traits.api \
     import Any, Bool, Delegate, Enum, Float, Instance, Int, List, \
@@ -20,6 +22,7 @@ coordinate_delegate = Delegate("inner", modify=True)
 
 DEFAULT_DRAWING_ORDER = ["background", "underlay", "mainlayer", "border", "overlay"]
 
+
 class Component(CoordinateBox, Interactor):
     """
     Component is the base class for most Enable objects.  In addition to the
@@ -313,8 +316,8 @@ class Component(CoordinateBox, Interactor):
     # into more than one class and that class
     classes = List
 
-    # The optional element ID of this component.
-    id = Str("")
+    # The element ID of this component.
+    id = Str
 
     # These will be used by the new layout system, but are currently unused.
     #max_width = Any
@@ -1027,7 +1030,6 @@ class Component(CoordinateBox, Interactor):
     def _get_layout_needed(self):
         return self._layout_needed
 
-
     def _tools_items_changed(self):
         self.invalidate_and_redraw()
 
@@ -1035,6 +1037,12 @@ class Component(CoordinateBox, Interactor):
     # Event handlers
     #------------------------------------------------------------------------
 
+    def _id_default(self):
+        """ Generate a random UUID for the ID.
+        """
+        # The first 32bits is plenty.
+        return uuid4().hex[:8]
+
     def _aspect_ratio_changed(self, old, new):
         if new is not None:
             self._enforce_aspect_ratio()
