@@ -34,7 +34,7 @@ class Window(BaseWindow):
         w = self._gc.width()
         h = self._gc.height()
         data = self._gc.pixel_map.convert_to_argb32string()
-        image = QtGui.QImage(data, w, h, QtGui.QImage.Format_RGB32)
+        image = QtGui.QImage(data, w, h, QtGui.QImage.Format_ARGB32)
         rect = QtCore.QRect(0,0,w,h)
         painter = QtGui.QPainter(self.control)
         painter.drawImage(rect, image)
