@@ -257,16 +257,16 @@ namespace kiva
             std::vector<gradient_stop>::iterator stop_it = this->stops.begin();
             double offset = 0.0;
             unsigned int i = 0;
+            unsigned int const array_size = array.size();
 
             for (; stop_it+1 != this->stops.end(); stop_it++)
             {
                 std::vector<gradient_stop>::iterator next_it = stop_it+1;
                 double offset_range = next_it->offset - stop_it->offset;
-                while ( (offset <= next_it->offset) && (offset <=1.0))
+                while ( (offset <= next_it->offset) && (i < array_size))
                 {
-                    array[i] = stop_it->color.gradient(next_it->color, (offset-stop_it->offset)/offset_range);
-                    i++;
-                    offset = i/double(array.size());
+                    array[i++] = stop_it->color.gradient(next_it->color, (offset-stop_it->offset)/offset_range);
+                    offset = i/double(array_size-1);
                 }
             }
         }
