@@ -1,6 +1,7 @@
 #!/usr/bin/env python
 
 import os
+import platform
 import sys
 
 def configuration(parent_package='', top_path=None):
@@ -28,67 +29,73 @@ def configuration(parent_package='', top_path=None):
                     cython_result.num_errors)
         return target
 
-    extra_link_args=[
-        '-Wl,-framework', '-Wl,CoreFoundation',
-        '-Wl,-framework', '-Wl,ApplicationServices',
-        '-Wl,-framework', '-Wl,Carbon',
-        '-Wl,-framework', '-Wl,Foundation',
+    frameworks = ['CoreFoundation','ApplicationServices','Foundation']
+    extra_link_args=['-framework %s' % x for x in frameworks]
+    include_dirs = [
+        '/System/Library/Frameworks/%s.framework/Versions/A/Headers' % x
+        for x in frameworks
     ]
-    include_dirs = ['/Developer/Headers/FlatCarbon']
-    config.add_extension('ATSFont',
-                         [generate_c_from_cython],
-                         include_dirs = include_dirs,
-                         extra_link_args = extra_link_args,
-                         depends=["ATSFont.pyx",
-                                  "Python.pxi",
-                                  "ATS.pxi",
-                                  ],
-                         )
+
     config.add_extension('ABCGI',
                          [generate_c_from_cython],
-                         include_dirs = include_dirs,
+                         extra_link_args = extra_link_args,
                          depends = ["ABCGI.pyx",
-                                    "ATSUI.pxi",
                                     "Python.pxi",
                                     "numpy.pxi",
                                     "c_numpy.pxd",
                                     "CoreFoundation.pxi",
                                     "CoreGraphics.pxi",
-                                    "QuickDraw.pxi",
+                                    "CoreText.pxi",
                                     ]
                          )
 
+    config.add_extension('CTFont',
+                         [generate_c_from_cython],
+                         extra_link_args = extra_link_args,
+                         depends=["CTFont.pyx",
+                                  "CoreFoundation.pxi",
+                                  "CoreGraphics.pxi",
+                                  "CoreText.pxi",
+                                  ],
+                        )
+
+    config.add_extension("mac_context",
+                         ["mac_context.c", "mac_context_cocoa.m"],
+                         include_dirs = include_dirs,
+                         extra_link_args = extra_link_args,
+                         depends = ["mac_context.h"],
+                         )
+
     wx_info = get_info('wx')
     if wx_info:
-        # Find the release number of wx.
         wx_release = '2.6'
         for macro, value in wx_info['define_macros']:
             if macro.startswith('WX_RELEASE_'):
                 wx_release = macro[len('WX_RELEASE_'):].replace('_', '.')
                 break
 
+        # only build macport for wxPython version 2.6, it's not needed in the
+        # newer releases (see __init__.py)
         if wx_release == '2.6':
             macport_cpp = config.paths('macport26.cpp')[0]
-        else:
-            macport_cpp = config.paths('macport28.cpp')[0]
 
-        def get_macport_cpp(extension, build_dir):
-            if sys.platform != 'darwin':
-                print 'No %s will be built for this platform.'%(extension.name)
-                return None
-
-            elif wx_release not in ('2.6', '2.8'):
-                print ('No %s will be built because we do not recognize '
-                       'wx version %s' % (extension.name, wx_release))
-                return None
-
-            return macport_cpp
-
-        info = {}
-        dict_append(info, define_macros=[("__WXMAC__", 1)])
-        dict_append(info, **wx_info)
-        config.add_extension('macport', [get_macport_cpp],
-                             depends = [macport_cpp],
-                             **wx_info
-                             )
+            def get_macport_cpp(extension, build_dir):
+                if sys.platform != 'darwin':
+                    print 'No %s will be built for this platform.'%(extension.name)
+                    return None
+    
+                elif wx_release not in ('2.6', '2.8'):
+                    print ('No %s will be built because we do not recognize '
+                           'wx version %s' % (extension.name, wx_release))
+                    return None
+    
+                return macport_cpp
+    
+            info = {}
+            dict_append(info, define_macros=[("__WXMAC__", 1)])
+            dict_append(info, **wx_info)
+            config.add_extension('macport', [get_macport_cpp],
+                                 depends = [macport_cpp],
+                                 **wx_info
+                                 )
     return config
